import os
import webbrowser
from flask import Flask, redirect, request, session, url_for
from requests_oauthlib import OAuth2Session
import requests
import secrets
import hashlib
import base64

app = Flask(__name__)

# Configuración de OAuth2
CLIENT_ID = 'SR_91uGHZXCp00zZcJ8A6'
CLIENT_SECRET = '3ZfhJrhmYVMOw51UFzVdKU6F1JeIMm9P8GSx8FpWOdtUJ3KduSxOHVNgOPeez5_fcF-5l3e0lg3c7yJw8WfkIA'
AUTHORIZATION_BASE_URL = 'https://login.pre.nautilus.app/oidc/authorize'
TOKEN_URL = 'https://login.pre.nautilus.app/oidc/oauth/token'
USERINFO_URL = 'https://login.pre.nautilus.app/oidc/userinfo'
REVOCATION_URL = 'https://login.pre.nautilus.app/oidc/revoke'
END_SESSION_ENDPOINT = 'https://login.pre.nautilus.app/oidc/session/end'
REDIRECT_URI = 'http://localhost:3000/cb'
SCOPE = ['openid', 'email', 'profile']

# Generar valores de seguridad
nonce = secrets.token_urlsafe(16)
state = secrets.token_urlsafe(16)
code_verifier = secrets.token_urlsafe(64)
code_challenge = base64.urlsafe_b64encode(hashlib.sha256(code_verifier.encode()).digest()).decode().rstrip('=')

@app.route('/')
def index():
    """Step 1: Redirect the user to the authorization URL."""
    oauth = OAuth2Session(CLIENT_ID, redirect_uri=REDIRECT_URI, scope=SCOPE, state=state)
    authorization_url, state = oauth.authorization_url(
        AUTHORIZATION_BASE_URL,
        nonce=nonce,
        code_challenge=code_challenge,
        code_challenge_method='S256'
    )

    # Guardar el estado en la sesión para usarlo en el callback
    session['oauth_state'] = state
    session['code_verifier'] = code_verifier
    return redirect(authorization_url)

@app.route('/cb')
def callback():
    """Step 2: Handle the callback from the authorization server."""
    oauth = OAuth2Session(CLIENT_ID, redirect_uri=REDIRECT_URI, state=session['oauth_state'])
    token = oauth.fetch_token(
        TOKEN_URL,
        client_secret=CLIENT_SECRET,
        authorization_response=request.url,
        code_verifier=session['code_verifier']
    )

    # Obtener información del usuario
    userinfo = oauth.get(USERINFO_URL).json()
    print('User Info:', userinfo)

    # Revocar el token de acceso
    revoke_token(token['access_token'])

    # Intentar obtener información del usuario después de la revocación
    try:
        userinfo_after_revoke = oauth.get(USERINFO_URL).json()
        print('User Info after revocation:', userinfo_after_revoke)
    except Exception as e:
        print('Failed to fetch user info after revocation:', e)

    # Cerrar sesión
    logout(token['id_token'])

    return f"Access Token: {token['access_token']}"

def revoke_token(access_token):
    response = requests.post(REVOCATION_URL, data={
        'token': access_token,
        'token_type_hint': 'access_token',
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET
    })
    if response.status_code == 200:
        print('Access token revoked successfully')
    else:
        print('Failed to revoke access token:', response.content)

def logout(id_token_hint):
    post_logout_redirect_uri = 'http://localhost:3000/post-logout'
    logout_url = f"{END_SESSION_ENDPOINT}?id_token_hint={id_token_hint}&post_logout_redirect_uri={post_logout_redirect_uri}"
    return redirect(logout_url)

@app.route('/post-logout')
def post_logout():
    print('You have been logged out successfully.')
    return 'You have been logged out successfully.'

if __name__ == '__main__':
    app.secret_key = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
    app.run(debug=True, port=3000)
    webbrowser.open('http://localhost:3000')
